@@ -86,7 +86,7 @@ if defined? Zlib
     def test_adler
       z = Zlib::Deflate.new
       z << "foo"
-      s = z.finish
+      z.finish
       assert_equal(0x02820145, z.adler)
     end
 
@@ -95,7 +95,7 @@ if defined? Zlib
       assert_equal(false, z.finished?)
       z << "foo"
       assert_equal(false, z.finished?)
-      s = z.finish
+      z.finish
       assert_equal(true, z.finished?)
       z.close
       assert_raise(Zlib::Error) { z.finished? }
@@ -195,6 +195,29 @@ if defined? Zlib
       z << "foo" # ???
     end
 
+    def test_inflate_dictionary
+      dictionary = "foo"
+
+      deflate = Zlib::Deflate.new
+      deflate.set_dictionary dictionary
+      compressed = deflate.deflate "foofoofoo", Zlib::FINISH
+      deflate.close
+
+      out = nil
+      inflate = Zlib::Inflate.new
+
+      begin
+        out = inflate.inflate compressed
+
+        flunk "Zlib::NeedDict was not raised"
+      rescue Zlib::NeedDict
+        inflate.set_dictionary dictionary
+        out = inflate.inflate ""
+      end
+
+      assert_equal "foofoofoo", out
+    end
+
     def test_sync
       z = Zlib::Deflate.new
       s = z.deflate("foo" * 1000, Zlib::FULL_FLUSH)
@@ -233,28 +256,30 @@ if defined? Zlib
 
   class TestZlibGzipFile < Test::Unit::TestCase
     def test_to_io
-      t = Tempfile.new("test_zlib_gzip_file")
+      t = Tempfile.new("test_zlib_gzip_file_to_io")
       t.close
       Zlib::GzipWriter.open(t.path) {|gz| gz.print("foo") }
 
-      f = Zlib::GzipReader.open(t.path)
-      assert_kind_of(IO, f.to_io)
+      Zlib::GzipReader.open(t.path) do |f|
+        assert_kind_of(IO, f.to_io)
+      end
     end
 
     def test_crc
-      t = Tempfile.new("test_zlib_gzip_file")
+      t = Tempfile.new("test_zlib_gzip_file_crc")
       t.close
       Zlib::GzipWriter.open(t.path) {|gz| gz.print("foo") }
 
-      f = Zlib::GzipReader.open(t.path)
-      f.read
-      assert_equal(0x8c736521, f.crc)
+      Zlib::GzipReader.open(t.path) do |f|
+        f.read
+        assert_equal(0x8c736521, f.crc)
+      end
     end
 
     def test_mtime
       tim = Time.now
 
-      t = Tempfile.new("test_zlib_gzip_file")
+      t = Tempfile.new("test_zlib_gzip_file_mtime")
       t.close
       Zlib::GzipWriter.open(t.path) do |gz|
         gz.mtime = -1
@@ -264,30 +289,33 @@ if defined? Zlib
         assert_raise(Zlib::GzipFile::Error) { gz.mtime = Time.now }
       end
 
-      f = Zlib::GzipReader.open(t.path)
-      assert_equal(tim.to_i, f.mtime.to_i)
+      Zlib::GzipReader.open(t.path) do |f|
+        assert_equal(tim.to_i, f.mtime.to_i)
+      end
     end
 
     def test_level
-      t = Tempfile.new("test_zlib_gzip_file")
+      t = Tempfile.new("test_zlib_gzip_file_level")
       t.close
       Zlib::GzipWriter.open(t.path) {|gz| gz.print("foo") }
 
-      f = Zlib::GzipReader.open(t.path)
-      assert_equal(Zlib::DEFAULT_COMPRESSION, f.level)
+      Zlib::GzipReader.open(t.path) do |f|
+        assert_equal(Zlib::DEFAULT_COMPRESSION, f.level)
+      end
     end
 
     def test_os_code
-      t = Tempfile.new("test_zlib_gzip_file")
+      t = Tempfile.new("test_zlib_gzip_file_os_code")
       t.close
       Zlib::GzipWriter.open(t.path) {|gz| gz.print("foo") }
 
-      f = Zlib::GzipReader.open(t.path)
-      assert_equal(Zlib::OS_CODE, f.os_code)
+      Zlib::GzipReader.open(t.path) do |f|
+        assert_equal(Zlib::OS_CODE, f.os_code)
+      end
     end
 
     def test_orig_name
-      t = Tempfile.new("test_zlib_gzip_file")
+      t = Tempfile.new("test_zlib_gzip_file_orig_name")
       t.close
       Zlib::GzipWriter.open(t.path) do |gz|
         gz.orig_name = "foobarbazqux\0quux"
@@ -296,12 +324,13 @@ if defined? Zlib
         assert_raise(Zlib::GzipFile::Error) { gz.orig_name = "quux" }
       end
 
-      f = Zlib::GzipReader.open(t.path)
-      assert_equal("foobarbazqux", f.orig_name)
+      Zlib::GzipReader.open(t.path) do |f|
+        assert_equal("foobarbazqux", f.orig_name)
+      end
     end
 
     def test_comment
-      t = Tempfile.new("test_zlib_gzip_file")
+      t = Tempfile.new("test_zlib_gzip_file_comment")
       t.close
       Zlib::GzipWriter.open(t.path) do |gz|
         gz.comment = "foobarbazqux\0quux"
@@ -310,52 +339,55 @@ if defined? Zlib
         assert_raise(Zlib::GzipFile::Error) { gz.comment = "quux" }
       end
 
-      f = Zlib::GzipReader.open(t.path)
-      assert_equal("foobarbazqux", f.comment)
+      Zlib::GzipReader.open(t.path) do |f|
+        assert_equal("foobarbazqux", f.comment)
+      end
     end
 
     def test_lineno
-      t = Tempfile.new("test_zlib_gzip_file")
+      t = Tempfile.new("test_zlib_gzip_file_lineno")
       t.close
       Zlib::GzipWriter.open(t.path) {|gz| gz.print("foo\nbar\nbaz\nqux\n") }
 
-      f = Zlib::GzipReader.open(t.path)
-      assert_equal([0, "foo\n"], [f.lineno, f.gets])
-      assert_equal([1, "bar\n"], [f.lineno, f.gets])
-      f.lineno = 1000
-      assert_equal([1000, "baz\n"], [f.lineno, f.gets])
-      assert_equal([1001, "qux\n"], [f.lineno, f.gets])
+      Zlib::GzipReader.open(t.path) do |f|
+        assert_equal([0, "foo\n"], [f.lineno, f.gets])
+        assert_equal([1, "bar\n"], [f.lineno, f.gets])
+        f.lineno = 1000
+        assert_equal([1000, "baz\n"], [f.lineno, f.gets])
+        assert_equal([1001, "qux\n"], [f.lineno, f.gets])
+      end
     end
 
     def test_closed_p
-      t = Tempfile.new("test_zlib_gzip_file")
+      t = Tempfile.new("test_zlib_gzip_file_closed_p")
       t.close
       Zlib::GzipWriter.open(t.path) {|gz| gz.print("foo") }
 
-      f = Zlib::GzipReader.open(t.path)
-      assert_equal(false, f.closed?)
-      f.read
-      assert_equal(false, f.closed?)
-      f.close
-      assert_equal(true, f.closed?)
+      Zlib::GzipReader.open(t.path) do |f|
+        assert_equal(false, f.closed?)
+        f.read
+        assert_equal(false, f.closed?)
+        f.close
+        assert_equal(true, f.closed?)
+      end
     end
 
     def test_sync
-      t = Tempfile.new("test_zlib_gzip_file")
+      t = Tempfile.new("test_zlib_gzip_file_sync")
       t.close
       Zlib::GzipWriter.open(t.path) {|gz| gz.print("foo") }
 
-      f = Zlib::GzipReader.open(t.path)
-      f.sync = true
-      assert_equal(true, f.sync)
-      f.read
-      f.sync = false
-      assert_equal(false, f.sync)
-      f.close
+      Zlib::GzipReader.open(t.path) do |f|
+        f.sync = true
+        assert_equal(true, f.sync)
+        f.read
+        f.sync = false
+        assert_equal(false, f.sync)
+      end
     end
 
     def test_pos
-      t = Tempfile.new("test_zlib_gzip_file")
+      t = Tempfile.new("test_zlib_gzip_file_pos")
       t.close
       Zlib::GzipWriter.open(t.path) do |gz|
         gz.print("foo")
@@ -365,7 +397,7 @@ if defined? Zlib
     end
 
     def test_path
-      t = Tempfile.new("test_zlib_gzip_file")
+      t = Tempfile.new("test_zlib_gzip_file_path")
       t.close
 
       gz = Zlib::GzipWriter.open(t.path)
@@ -374,10 +406,11 @@ if defined? Zlib
       gz.close
       assert_equal(t.path, gz.path)
 
-      f = Zlib::GzipReader.open(t.path)
-      assert_equal(t.path, f.path)
-      f.close
-      assert_equal(t.path, f.path)
+      Zlib::GzipReader.open(t.path) do |f|
+        assert_equal(t.path, f.path)
+        f.close
+        assert_equal(t.path, f.path)
+      end
 
       s = ""
       sio = StringIO.new(s)
@@ -387,9 +420,9 @@ if defined? Zlib
       gz.close
 
       sio = StringIO.new(s)
-      f = Zlib::GzipReader.new(sio)
-      assert_raise(NoMethodError) { f.path }
-      f.close
+      Zlib::GzipReader.new(sio) do |f|
+        assert_raise(NoMethodError) { f.path }
+      end
     end
   end
 
@@ -429,8 +462,26 @@ if defined? Zlib
     end
 
     def test_open
-      t = Tempfile.new("test_zlib_gzip_reader")
+      t = Tempfile.new("test_zlib_gzip_reader_open")
       t.close
+      e = assert_raise(Zlib::GzipFile::Error) {
+        Zlib::GzipReader.open(t.path)
+      }
+      assert_equal("not in gzip format", e.message)
+      assert_nil(e.input)
+      open(t.path, "wb") {|f| f.write("foo")}
+      e = assert_raise(Zlib::GzipFile::Error) {
+        Zlib::GzipReader.open(t.path)
+      }
+      assert_equal("not in gzip format", e.message)
+      assert_equal("foo", e.input)
+      open(t.path, "wb") {|f| f.write("foobarzothoge")}
+      e = assert_raise(Zlib::GzipFile::Error) {
+        Zlib::GzipReader.open(t.path)
+      }
+      assert_equal("not in gzip format", e.message)
+      assert_equal("foobarzothoge", e.input)
+
       Zlib::GzipWriter.open(t.path) {|gz| gz.print("foo") }
 
       assert_raise(ArgumentError) { Zlib::GzipReader.open }
@@ -438,162 +489,248 @@ if defined? Zlib
       assert_equal("foo", Zlib::GzipReader.open(t.path) {|gz| gz.read })
 
       f = Zlib::GzipReader.open(t.path)
-      assert_equal("foo", f.read)
-      f.close
+      begin
+        assert_equal("foo", f.read)
+      ensure
+        f.close
+      end
     end
 
     def test_rewind
-      t = Tempfile.new("test_zlib_gzip_reader")
+      t = Tempfile.new("test_zlib_gzip_reader_rewind")
       t.close
       Zlib::GzipWriter.open(t.path) {|gz| gz.print("foo") }
 
-      f = Zlib::GzipReader.open(t.path)
-      assert_equal("foo", f.read)
-      f.rewind
-      assert_equal("foo", f.read)
-      f.close
+      Zlib::GzipReader.open(t.path) do |f|
+        assert_equal("foo", f.read)
+        f.rewind
+        assert_equal("foo", f.read)
+      end
     end
 
     def test_unused
-      t = Tempfile.new("test_zlib_gzip_reader")
+      t = Tempfile.new("test_zlib_gzip_reader_unused")
       t.close
       Zlib::GzipWriter.open(t.path) {|gz| gz.print("foobar") }
 
-      f = Zlib::GzipReader.open(t.path)
-      assert_equal("foo", f.read(3))
-      f.unused
-      assert_equal("bar", f.read)
-      f.unused
-      f.close
+      Zlib::GzipReader.open(t.path) do |f|
+        assert_equal(nil, f.unused)
+        assert_equal("foo", f.read(3))
+        assert_equal(nil, f.unused)
+        assert_equal("bar", f.read)
+        assert_equal(nil, f.unused)
+      end
+    end
+
+    def test_unused2
+      zio = StringIO.new
+
+      io = Zlib::GzipWriter.new zio
+      io.write 'aaaa'
+      io.finish
+
+      io = Zlib::GzipWriter.new zio
+      io.write 'bbbb'
+      io.finish
+
+      zio.rewind
+
+      io = Zlib::GzipReader.new zio
+      assert_equal('aaaa', io.read)
+      unused = io.unused
+      assert_equal(24, unused.bytesize)
+      io.finish
+
+      zio.pos -= unused.length
+
+      io = Zlib::GzipReader.new zio
+      assert_equal('bbbb', io.read)
+      assert_equal(nil, io.unused)
+      io.finish
     end
 
     def test_read
-      t = Tempfile.new("test_zlib_gzip_reader")
+      t = Tempfile.new("test_zlib_gzip_reader_read")
       t.close
       str = "\u3042\u3044\u3046"
       Zlib::GzipWriter.open(t.path) {|gz| gz.print(str) }
 
-      f = Zlib::GzipReader.open(t.path, encoding: "UTF-8")
-      assert_raise(ArgumentError) { f.read(-1) }
-      assert_equal(str, f.read)
+      Zlib::GzipReader.open(t.path, encoding: "UTF-8") do |f|
+        assert_raise(ArgumentError) { f.read(-1) }
+        assert_equal(str, f.read)
+      end
     end
 
     def test_readpartial
-      t = Tempfile.new("test_zlib_gzip_reader")
+      t = Tempfile.new("test_zlib_gzip_reader_readpartial")
       t.close
       Zlib::GzipWriter.open(t.path) {|gz| gz.print("foobar") }
 
-      f = Zlib::GzipReader.open(t.path)
-      assert("foo".start_with?(f.readpartial(3)))
+      Zlib::GzipReader.open(t.path) do |f|
+        assert("foo".start_with?(f.readpartial(3)))
+      end
 
-      f = Zlib::GzipReader.open(t.path)
-      s = ""
-      f.readpartial(3, s)
-      assert("foo".start_with?(s))
+      Zlib::GzipReader.open(t.path) do |f|
+        s = ""
+        f.readpartial(3, s)
+        assert("foo".start_with?(s))
 
-      assert_raise(ArgumentError) { f.readpartial(-1) }
+        assert_raise(ArgumentError) { f.readpartial(-1) }
+      end
     end
 
     def test_getc
-      t = Tempfile.new("test_zlib_gzip_reader")
+      t = Tempfile.new("test_zlib_gzip_reader_getc")
       t.close
       Zlib::GzipWriter.open(t.path) {|gz| gz.print("foobar") }
 
-      f = Zlib::GzipReader.open(t.path)
-      "foobar".each_char {|c| assert_equal(c, f.getc) }
-      assert_nil(f.getc)
+      Zlib::GzipReader.open(t.path) do |f|
+        "foobar".each_char {|c| assert_equal(c, f.getc) }
+        assert_nil(f.getc)
+      end
     end
 
     def test_getbyte
-      t = Tempfile.new("test_zlib_gzip_reader")
+      t = Tempfile.new("test_zlib_gzip_reader_getbyte")
       t.close
       Zlib::GzipWriter.open(t.path) {|gz| gz.print("foobar") }
 
-      f = Zlib::GzipReader.open(t.path)
-      "foobar".each_byte {|c| assert_equal(c, f.getbyte) }
-      assert_nil(f.getbyte)
+      Zlib::GzipReader.open(t.path) do |f|
+        "foobar".each_byte {|c| assert_equal(c, f.getbyte) }
+        assert_nil(f.getbyte)
+      end
     end
 
     def test_readchar
-      t = Tempfile.new("test_zlib_gzip_reader")
+      t = Tempfile.new("test_zlib_gzip_reader_readchar")
       t.close
       Zlib::GzipWriter.open(t.path) {|gz| gz.print("foobar") }
 
-      f = Zlib::GzipReader.open(t.path)
-      "foobar".each_byte {|c| assert_equal(c, f.readchar.ord) }
-      assert_raise(EOFError) { f.readchar }
+      Zlib::GzipReader.open(t.path) do |f|
+        "foobar".each_byte {|c| assert_equal(c, f.readchar.ord) }
+        assert_raise(EOFError) { f.readchar }
+      end
     end
 
     def test_each_byte
-      t = Tempfile.new("test_zlib_gzip_reader")
+      t = Tempfile.new("test_zlib_gzip_reader_each_byte")
       t.close
       Zlib::GzipWriter.open(t.path) {|gz| gz.print("foobar") }
 
-      f = Zlib::GzipReader.open(t.path)
-      a = []
-      f.each_byte {|c| a << c }
-      assert_equal("foobar".each_byte.to_a, a)
+      Zlib::GzipReader.open(t.path) do |f|
+        a = []
+        f.each_byte {|c| a << c }
+        assert_equal("foobar".each_byte.to_a, a)
+      end
     end
 
     def test_gets
-      t = Tempfile.new("test_zlib_gzip_reader")
+      t = Tempfile.new("test_zlib_gzip_reader_gets")
       t.close
       Zlib::GzipWriter.open(t.path) {|gz| gz.print("foo\nbar\nbaz\n") }
 
-      f = Zlib::GzipReader.open(t.path)
-      assert_equal("foo\n", f.gets)
-      assert_equal("bar\n", f.gets)
-      assert_equal("baz\n", f.gets)
-      assert_nil(f.gets)
-      f.close
+      Zlib::GzipReader.open(t.path) do |f|
+        assert_equal("foo\n", f.gets)
+        assert_equal("bar\n", f.gets)
+        assert_equal("baz\n", f.gets)
+        assert_nil(f.gets)
+      end
 
-      f = Zlib::GzipReader.open(t.path)
-      assert_equal("foo\nbar\nbaz\n", f.gets(nil))
-      f.close
+      Zlib::GzipReader.open(t.path) do |f|
+        assert_equal("foo\nbar\nbaz\n", f.gets(nil))
+      end
+
+      Zlib::GzipReader.open(t.path) do |f|
+        assert_equal("foo\n", f.gets(10))
+        assert_equal("ba", f.gets(2))
+        assert_equal("r\nb", f.gets(nil, 3))
+        assert_equal("az\n", f.gets(nil, 10))
+        assert_nil(f.gets)
+      end
     end
 
-    def test_gets
-      t = Tempfile.new("test_zlib_gzip_reader")
+    def test_gets2
+      t = Tempfile.new("test_zlib_gzip_reader_gets2")
+      t.close
+      ustrs = %W"\u{3042 3044 3046}\n \u{304b 304d 304f}\n \u{3055 3057 3059}\n"
+      Zlib::GzipWriter.open(t.path) {|gz| gz.print(*ustrs) }
+
+      Zlib::GzipReader.open(t.path, encoding: "UTF-8") do |f|
+        assert_equal(ustrs[0], f.gets)
+        assert_equal(ustrs[1], f.gets)
+        assert_equal(ustrs[2], f.gets)
+        assert_nil(f.gets)
+      end
+
+      Zlib::GzipReader.open(t.path, encoding: "UTF-8") do |f|
+        assert_equal(ustrs.join(''), f.gets(nil))
+      end
+
+      Zlib::GzipReader.open(t.path, encoding: "UTF-8") do |f|
+        assert_equal(ustrs[0], f.gets(20))
+        assert_equal(ustrs[1][0,2], f.gets(5))
+        assert_equal(ustrs[1][2..-1]+ustrs[2][0,1], f.gets(nil, 5))
+        assert_equal(ustrs[2][1..-1], f.gets(nil, 20))
+        assert_nil(f.gets)
+      end
+    end
+
+    def test_readline
+      t = Tempfile.new("test_zlib_gzip_reader_readline")
       t.close
       Zlib::GzipWriter.open(t.path) {|gz| gz.print("foo\nbar\nbaz\n") }
 
-      f = Zlib::GzipReader.open(t.path)
-      assert_equal("foo\n", f.readline)
-      assert_equal("bar\n", f.readline)
-      assert_equal("baz\n", f.readline)
-      assert_raise(EOFError) { f.readline }
-      f.close
+      Zlib::GzipReader.open(t.path) do |f|
+        assert_equal("foo\n", f.readline)
+        assert_equal("bar\n", f.readline)
+        assert_equal("baz\n", f.readline)
+        assert_raise(EOFError) { f.readline }
+      end
     end
 
     def test_each
-      t = Tempfile.new("test_zlib_gzip_reader")
+      t = Tempfile.new("test_zlib_gzip_reader_each")
       t.close
       Zlib::GzipWriter.open(t.path) {|gz| gz.print("foo\nbar\nbaz\n") }
 
-      f = Zlib::GzipReader.open(t.path)
-      a = ["foo\n", "bar\n", "baz\n"]
-      f.each {|l| assert_equal(a.shift, l) }
-      f.close
+      Zlib::GzipReader.open(t.path) do |f|
+        a = ["foo\n", "bar\n", "baz\n"]
+        f.each {|l| assert_equal(a.shift, l) }
+      end
     end
 
     def test_readlines
-      t = Tempfile.new("test_zlib_gzip_reader")
+      t = Tempfile.new("test_zlib_gzip_reader_readlines")
       t.close
       Zlib::GzipWriter.open(t.path) {|gz| gz.print("foo\nbar\nbaz\n") }
 
-      f = Zlib::GzipReader.open(t.path)
-      assert_equal(["foo\n", "bar\n", "baz\n"], f.readlines)
-      f.close
+      Zlib::GzipReader.open(t.path) do |f|
+        assert_equal(["foo\n", "bar\n", "baz\n"], f.readlines)
+      end
     end
 
     def test_reader_wrap
-      t = Tempfile.new("test_zlib_gzip_reader")
+      t = Tempfile.new("test_zlib_gzip_reader_wrap")
       t.close
       Zlib::GzipWriter.open(t.path) {|gz| gz.print("foo") }
       f = open(t.path)
       assert_equal("foo", Zlib::GzipReader.wrap(f) {|gz| gz.read })
       assert_raise(IOError) { f.close }
     end
+
+    def test_corrupted_header
+      gz = Zlib::GzipWriter.new(StringIO.new(s = ""))
+      gz.orig_name = "X"
+      gz.comment = "Y"
+      gz.print("foo")
+      gz.finish
+      # 14: magic(2) + method(1) + flag(1) + mtime(4) + exflag(1) + os(1) + orig_name(2) + comment(2)
+      1.upto(14) do |idx|
+        assert_raise(Zlib::GzipFile::Error, idx) do
+          Zlib::GzipReader.new(StringIO.new(s[0, idx])).read
+        end
+      end
+    end
   end
 
   class TestZlibGzipWriter < Test::Unit::TestCase
@@ -607,21 +744,24 @@ if defined? Zlib
     def test_open
       assert_raise(ArgumentError) { Zlib::GzipWriter.open }
 
-      t = Tempfile.new("test_zlib_gzip_writer")
+      t = Tempfile.new("test_zlib_gzip_writer_open")
       t.close
       Zlib::GzipWriter.open(t.path) {|gz| gz.print("foo") }
       assert_equal("foo", Zlib::GzipReader.open(t.path) {|gz| gz.read })
 
       f = Zlib::GzipWriter.open(t.path)
-      f.print("bar")
-      f.close
+      begin
+        f.print("bar")
+      ensure
+        f.close
+      end
       assert_equal("bar", Zlib::GzipReader.open(t.path) {|gz| gz.read })
 
       assert_raise(Zlib::StreamError) { Zlib::GzipWriter.open(t.path, 10000) }
     end
 
     def test_write
-      t = Tempfile.new("test_zlib_gzip_writer")
+      t = Tempfile.new("test_zlib_gzip_writer_write")
       t.close
       Zlib::GzipWriter.open(t.path) {|gz| gz.print("foo") }
       assert_equal("foo", Zlib::GzipReader.open(t.path) {|gz| gz.read })
@@ -633,7 +773,7 @@ if defined? Zlib
     end
 
     def test_putc
-      t = Tempfile.new("test_zlib_gzip_writer")
+      t = Tempfile.new("test_zlib_gzip_writer_putc")
       t.close
       Zlib::GzipWriter.open(t.path) {|gz| gz.putc(?x) }
       assert_equal("x", Zlib::GzipReader.open(t.path) {|gz| gz.read })
@@ -642,7 +782,7 @@ if defined? Zlib
     end
 
     def test_writer_wrap
-      t = Tempfile.new("test_zlib_gzip_writer")
+      t = Tempfile.new("test_zlib_gzip_writer_wrap")
       Zlib::GzipWriter.wrap(t) {|gz| gz.print("foo") }
       t.close
       assert_equal("foo", Zlib::GzipReader.open(t.path) {|gz| gz.read })
@@ -662,6 +802,16 @@ if defined? Zlib
       assert_equal(0x8a62c964, Zlib.adler32("abc\x01\x02\x03" * 10000))
     end
 
+    def test_adler32_combine
+      one = Zlib.adler32("fo")
+      two = Zlib.adler32("o")
+      begin
+        assert_equal(0x02820145, Zlib.adler32_combine(one, two, 1))
+      rescue NotImplementedError
+        skip "adler32_combine is not implemented"
+      end
+    end
+
     def test_crc32
       assert_equal(0x00000000, Zlib.crc32)
       assert_equal(0x8c736521, Zlib.crc32("foo"))
@@ -669,6 +819,16 @@ if defined? Zlib
       assert_equal(0x07f0d68f, Zlib.crc32("abc\x01\x02\x03" * 10000))
     end
 
+    def test_crc32_combine
+      one = Zlib.crc32("fo")
+      two = Zlib.crc32("o")
+      begin
+        assert_equal(0x8c736521, Zlib.crc32_combine(one, two, 1))
+      rescue NotImplementedError
+        skip "crc32_combine is not implemented"
+      end
+    end
+
     def test_crc_table
       t = Zlib.crc_table
       assert_instance_of(Array, t)
